//==============================================================================
//    Guangzhou Aosong
//==============================================================================
// Project   :  AFM Sample Code (V1.0)
// File      :  afm.c (V1.0)
// Author    :  RFU
// Date      :  2020-04-01
// Controller:  STM32F100RB
// IDE       :  Vision V4.60.0.0
// Compiler  :  Armcc
// Brief     :  Sensor Layer: Implementation of functions for sensor access.
//==============================================================================

//-- Includes ------------------------------------------------------------------
#include "afm.h"
#include "i2c_hal.h"

extern void Delay_Nms(uint32_t ms);	
unsigned short	device_status;

//-- Global Variables ----------------------------------------------------------
u16t currentCommand = 0x0000;


void DelayMicroSeconds(u32t nbrOfUs)
{
	uint32_t k;

	while (nbrOfUs--)
	{
		for (k = 0; k < 50; k++);	// 50
	}
}

//==============================================================================
void AFM_Init(void){
//==============================================================================
  I2c_Init(); // init I2C
}

//==============================================================================
etError AFM_WriteCommand(etCommands cmd){
//==============================================================================
	etError error; // error code

	// write command to sensor
	I2c_StartCondition();
	error  = I2c_WriteByte(I2C_ADR << 1 | I2C_WRITE);
	error |= I2c_WriteByte(cmd >> 8);
	error |= I2c_WriteByte(cmd & 0xFF);
	I2c_StopCondition();

	// if no error, store current command
	if(error == NO_ERROR)
		currentCommand = cmd;

	return error;
}

//==============================================================================
etError AFM_ReadCommandResult(u16t *result){
//==============================================================================
	etError error;    // error code
	u8t     checksum; // checksum byte
	u8t     data[2];  // read data array

	// read command result & checksum from sensor
	I2c_StartCondition();
	error    = I2c_WriteByte(I2C_ADR << 1 | I2C_READ);
	data[0]  = I2c_ReadByte(ACK);
	data[1]  = I2c_ReadByte(ACK);
	checksum = I2c_ReadByte(NO_ACK);
	I2c_StopCondition();

	// checksum verification
	error  |= AFM_CheckCrc (data, 2, checksum);

	// if no error, combine 16-bit result from the read data array
	if(error == NO_ERROR)
		*result = (data[0] << 8) | data[1];

	return error;
}

//==============================================================================
etError AFM_ReadCommandResultWithTimeout(u8t maxRetries, u16t *result){
//==============================================================================
	etError error; //variable for error code

	while(maxRetries--)
	{
		// try to read command result
		error = AFM_ReadCommandResult(result); 

		// if read command result was successful -> exit loop
		// it will only be successful if a new valid measurement was performed
		if(error == NO_ERROR) 
			break;

		// if it was not successful -> wait a short time and then try it again
		DelayMicroSeconds(200);		// 1000
	}

	return error;
}

//==============================================================================
etError AFM_GetFlow(ft offset, ft scale, ft *flow){
//==============================================================================
	etError error = NO_ERROR; // error code
	u16t    result;           // read result from sensor

	// write command if it is not already set 
	// if(currentCommand != FLOW_MEASUREMENT)
	error = AFM_WriteCommand(FLOW_MEASUREMENT);	

	// if no error, read command result
	// if(error == NO_ERROR)
	error = AFM_ReadCommandResultWithTimeout(4, &result);		// 20

	// if no error, compute the flow
	if(error == NO_ERROR)
		*flow = ((ft)result - offset) / scale;

	return error;
}

//==============================================================================
etError AFM_GetSerialNumber(u32t *serialNumber){
//==============================================================================
	etError error = NO_ERROR; // error code
	u16t result;              // read result from sensor

	// write command "read serial number (bit 31:16)"
	error = AFM_WriteCommand(READ_SERIAL_NUMBER_HIGH);

	// if no error, read command result
	if(error == NO_ERROR)
		error = AFM_ReadCommandResult(&result);

	// if no error, copy upper 16 bits to serial number
	if(error == NO_ERROR)
		*serialNumber = result << 16;

	// if no error, write command "read serial number (bit 15:0)"
	if(error == NO_ERROR)
		error = AFM_WriteCommand(READ_SERIAL_NUMBER_LOW);

	// if no error, read command result
	if(error == NO_ERROR)
		error = AFM_ReadCommandResult(&result);

	// if no error, copy lower 16 bits to serial number
	if(error == NO_ERROR)
		*serialNumber |= result;

	return error;
}

//==============================================================================
etError AFM_SoftReset(void){
//==============================================================================
	etError error; // error code

	error = AFM_WriteCommand(SOFT_RESET);

	return error;
}

//==============================================================================
etError AFM_CheckCrc(u8t data[], u8t nbrOfBytes, u8t checksum){
//==============================================================================
	u8t bit;     // bit mask
	u8t crc = 0; // calculated checksum
	u8t byteCtr; // byte counter

	// calculates 8-Bit checksum with given polynomial
	for(byteCtr = 0; byteCtr < nbrOfBytes; byteCtr++)
	{
		crc ^= (data[byteCtr]);
		for(bit = 8; bit > 0; --bit)
		{
			  if(crc & 0x80) 
				  crc = (crc << 1) ^ POLYNOMIAL;
			  else           
				  crc = (crc << 1);
		}
	}

	// verify checksum
	if(crc != checksum) 
		return CHECKSUM_ERROR;
	else                
		return NO_ERROR;

}
